import {
  ClassType,
} from '@loafer/core/constants';

export type MetadataDefinable<T> = {new(clazz: ClassType): T};

const getClassMetadata = <DefinitionType>(clazz: ClassType,
                                          metadataKey: any,
                                          definitionType: MetadataDefinable<DefinitionType>,
                                          isCreate: boolean = false): DefinitionType => {

  let definition: DefinitionType;
  if (Reflect.hasOwnMetadata(metadataKey, clazz) !== true) {
      if (!isCreate) {
          return undefined;
      }
      definition = Object.create(definitionType.prototype);
      definition.constructor.call(definition, clazz);

      Reflect.defineMetadata(metadataKey, definition, clazz);
  } else {
    definition = Reflect.getMetadata(metadataKey, clazz);
  }
  return definition;
};

export default getClassMetadata;
