import {
  ClassType,
  DecoratorType,
  DESIGN_PARAMTYPES,
  DESIGN_TYPE,
  PropertyType,
} from '@loafer/core/constants';
import { InjectSterotype } from '@loafer/pouches/constants';
import { validateQualifier } from '@loafer/pouches/util/qualifier';
import InjectConfig from '@loafer/pouches/decorator/InjectConfig';

export interface InjectItem {
  stereotype: InjectSterotype;
  decoratorType: DecoratorType;
  propertyConfig?: InjectConfig;
  parameterConfigMap?: Map<number, InjectConfig>;
}

class InjectDefinition {
  protected clazz: ClassType;
  protected injectMap: Map<PropertyType, InjectItem>;

  public constructor(clazz: ClassType) {
    this.clazz = clazz;
    this.injectMap = new Map();
  }

  public get injectors(): Map<PropertyType, InjectItem> {
    return this.injectMap;
  }

  /**
   * addInject
   */
  public addInject(injectConfig: InjectConfig, propertyKey: PropertyType, parameterIndex?: number): void {
    injectConfig.qualifier = validateQualifier(this.clazz, injectConfig.qualifier);
    if (undefined === injectConfig.required) {
      injectConfig.required = false;
    }

    if(typeof parameterIndex === 'number') {
      this.addInjectParameter(injectConfig, propertyKey, parameterIndex);
    } else {
      this.addInjectProperty(injectConfig, propertyKey);
    }
  }

  /**
   * addValue
   */
  public addValue(value: string, propertyKey: PropertyType, parameterIndex?: number): void {
    if(typeof parameterIndex === 'number') {
      this.addValueParameter(value, propertyKey, parameterIndex);
    } else {
      this.addValueProperty(value, propertyKey);
    }
  }

  private addValueProperty(value: string, propertyKey: PropertyType): void {
    return;
  }
  private addValueParameter(value: string, propertyKey: PropertyType, parameterIndex: number): void {
    return;
  }



  private addInjectParameter(injectConfig: InjectConfig,
                             propertyKey: PropertyType, parameterIndex: number): void {
    let injectItem: InjectItem;
    let parameterTypes: any[] = Reflect.getMetadata(DESIGN_PARAMTYPES, this.clazz, propertyKey);

    if (undefined === injectConfig.clazz) {
      injectConfig.clazz = parameterTypes[parameterIndex];
    }

    if (this.injectMap.has(propertyKey)) {
      injectItem = this.injectMap.get(propertyKey);
    } else {
      injectItem = {
        stereotype: InjectSterotype.INJECT,
        decoratorType: DecoratorType.PARAMETER,
      };
      this.injectMap.set(propertyKey, injectItem);
    }

    if (null !== injectItem.parameterConfigMap) {
      if (injectItem.parameterConfigMap.has(parameterIndex)) {
        throw new Error(`Cannot apply @inject decorator on one parameter[${propertyKey}:${parameterIndex}] multiple times.`);
      }
    } else {
      injectItem.parameterConfigMap = new Map();
    }

    injectItem.parameterConfigMap.set(parameterIndex, injectConfig);

  }

  private addInjectProperty(injectConfig: InjectConfig, propertyKey: PropertyType): void {
    if (this.injectMap.has(propertyKey)) {
      throw new Error(`Cannot apply @inject decorator on one property[${propertyKey}] multiple times.`);
    }

    let propertyType: any = Reflect.getMetadata(DESIGN_TYPE, this.clazz, propertyKey);

    if (undefined === injectConfig.clazz) {
      injectConfig.clazz = propertyType;
    }

    let injectItem: InjectItem = {
      stereotype: InjectSterotype.INJECT,
      decoratorType: DecoratorType.PROPERTY,
      propertyConfig: injectConfig,
    };

    this.injectMap.set(propertyKey, injectItem);
  }

}

export default InjectDefinition;
