import {
  ClassType,
  PropertyType,
} from '@loafer/core/constants';

import {
  validateQualifier,
} from '@loafer/pouches/util/qualifier';

import PouchDefinition from '@loafer/pouches/factory/config/PouchDefinition';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';
import DefaultSingletonPouchRegistry from '@loafer/pouches/factory/implement/DefaultSingletonPouchRegistry';



abstract class AbstractPouchFactory extends DefaultSingletonPouchRegistry implements PouchDefinitionRegistry {
  protected pouchDefinitionMap: Map<PropertyType, Map<ClassType, PouchDefinition>>;
  public constructor() {
    super();
    this.pouchDefinitionMap = new Map();
  }

  public registerPouchDefinition(pouchDefinition: PouchDefinition): void {
    let clazz = pouchDefinition.Clazz;
    let qualifier = validateQualifier(clazz, pouchDefinition.Qualifier);

    if (this.hasPouchDefinition(clazz, qualifier)) {
      throw new Error(`Pouch Definition[${clazz.constructor.name}:${qualifier}] is exist already`);
    }
    let map = this.pouchDefinitionMap.get(qualifier);
    if (undefined === map) {
      map = new Map();
      this.pouchDefinitionMap.set(qualifier, map);
    }
    map.set(clazz, pouchDefinition);
  }
  public removePouchDefinition(clazz: ClassType, qualifier?: PropertyType): void {
    const _qualifier = validateQualifier(clazz, qualifier);
    if (!this.hasPouchDefinition(clazz, _qualifier)) {
      throw new Error(`Pouch Definition[${clazz.constructor.name}:${qualifier}] is not exist`);
    }
    this.pouchDefinitionMap.get(_qualifier).delete(clazz);
    if (0 === this.pouchDefinitionMap.get(_qualifier).size) {
      this.pouchDefinitionMap.delete(_qualifier);
    }
  }
  public getPouchDefinition(clazz: ClassType, qualifier?: PropertyType): PouchDefinition {
    const _qualifier = validateQualifier(clazz, qualifier);

    if (!this.pouchDefinitionMap.has(_qualifier)) {
      return undefined;
    }
    return this.pouchDefinitionMap.get(_qualifier).get(clazz);
  }
  public hasPouchDefinition(clazz: ClassType, qualifier?: PropertyType): boolean {
    return undefined === this.getPouchDefinition(clazz, qualifier) ? false : true;
  }



}

export default AbstractPouchFactory;
